Imports System.Security.Principal
Imports System.DirectoryServices
Imports System.Runtime.InteropServices
Imports Microsoft.Interop.Security.AzRoles
Imports System.Configuration


Public Class AdamPrincipal
  Implements IPrincipal

  Private Shared m_app As IAzApplication
  Private Shared m_store As AzAuthorizationStoreClass
  Private ReadOnly m_IAzClientContext As IAzClientContext

  Private m_Identity As AdamIdentity
  Private m_roles() As Object

  <DllImport("advapi32.dll", SetLastError:=True, CharSet:=CharSet.Auto, EntryPoint:="ConvertSidToStringSid")> _
    Private Shared Function pf_ConvertSidToStringSid( _
    ByVal pSID As IntPtr, <[In](), Out(), MarshalAs(UnmanagedType.LPTStr)> ByRef pStringSid As String) As Boolean

  End Function
  Shared Sub New()
    Dim AuthZStorePath As Object = ConfigurationSettings.AppSettings("AuthZStorePath")
    If AuthZStorePath Is Nothing Then Throw New Exception("Can't find AuthZStorePath in the config file")
    m_store = New AzAuthorizationStoreClass
    m_store.Initialize(0, AuthZStorePath.ToString, Nothing)
    m_app = m_store.OpenApplication( _
     "app1", Nothing)
  End Sub

  Private Sub New(ByVal p_AdamIdentity As AdamIdentity, _
    ByVal p_IAzClientContext As IAzClientContext, _
    ByVal p_roles() As Object)
    m_roles = p_roles
    m_IAzClientContext = p_IAzClientContext
    m_Identity = p_AdamIdentity
  End Sub



  Public Function AccessCheck(ByVal p_operations() As Object, _
    Optional ByVal p_objectname As String = "Unspecified") As Boolean
    Dim scopes() As Object = {""}
    Dim results() As Object = CType(m_IAzClientContext.AccessCheck(p_objectname, _
                  scopes, p_operations), Object())
    For Each l_res As Object In results
      If Not CType(l_res, Integer) = 0 Then
        Return False
      End If
    Next
    Return True
  End Function

  Private Shared Function pf_makeFullDN(ByVal p_Name) As String
    Dim FullAdamUserStructure As Object = ConfigurationSettings.AppSettings("FullAdamUserStructure")
    If FullAdamUserStructure Is Nothing Then Throw New Exception("Can't find FullAdamUserStructure in the config file")
    Return String.Format(FullAdamUserStructure.ToString, p_Name)
  End Function

  Public Shared Function Login(ByVal p_Name As String, ByVal p_PWD As String) As AdamPrincipal
    Try
      Dim ADAMPath As Object = ConfigurationSettings.AppSettings("ADAMPath")
      If ADAMPath Is Nothing Then Throw New Exception("Can't find ADAMPath in the config file")
      Dim l_entry As New _
            DirectoryEntry(ADAMPath.ToString, _
            pf_makeFullDN(p_Name), p_PWD, _
            AuthenticationTypes.None)
      Dim mySearcher As New DirectorySearcher(l_entry)
      mySearcher.Filter = ("(&(objectClass=user)(name=" + p_Name + "))")
      Dim l_s As SearchResult = mySearcher.FindOne
      If l_s Is Nothing Then Throw New Exception("Critical Error, Couldn't find logged user")
      Dim SID As Byte() = l_s.Properties.Item("objectSID").Item(0)
      Dim sSID As String
      Dim sidPtr As IntPtr = Marshal.AllocHGlobal(SID.Length)
      Marshal.Copy(SID, 0, sidPtr, SID.Length)
      pf_ConvertSidToStringSid(CType(sidPtr, IntPtr), sSID)
      Dim l_IAzClientContext As IAzClientContext = m_app.InitializeClientContextFromStringSid(sSID, tagAZ_PROP_CONSTANTS.AZ_CLIENT_CONTEXT_SKIP_GROUP)
      Dim l_roles() As Object = l_IAzClientContext.GetRoles()
      Return New AdamPrincipal(New AdamIdentity(p_Name, l_s.GetDirectoryEntry), l_IAzClientContext, _
      l_roles)
    Catch ex As Exception
      Throw (New Exception("Login Failed"))
    End Try
  End Function

  Public ReadOnly Property Identity() As System.Security.Principal.IIdentity Implements System.Security.Principal.IPrincipal.Identity
    Get
      Return m_Identity
    End Get
  End Property

  Public Function IsInRole(ByVal role As String) As Boolean Implements System.Security.Principal.IPrincipal.IsInRole

  End Function
End Class
